<?php
/**
 * Plugin Name: Google Meet for LatePoint
 * Plugin URI:  https://1.envato.market/google-meet-latepoint
 * Description: LatePoint addon which allows you to automatically generate Google Meet meetings for your appointments.
 * Version:     9.1.0
 * Author:      TechXela
 * Author URI:  http://techxela.com
 * License:     https://codecanyon.net/licenses/standard
 * Text Domain: google-meet-latepoint
 * Domain Path: /languages
 */

/*
 * Google Meet for LatePoint
 * Copyright (c) 2022-2023 TechXela (https://codecanyon.net/user/tech-xela). All Rights Reserved.
 *
 * LICENSE
 * -------
 * This software is furnished under license(s) and may be used and copied only in accordance with the terms of such
 * license(s) along with the inclusion of the above copyright notice. If you purchased this software through CodeCanyon,
 * please read the full license(s) at: https://codecanyon.net/licenses/standard
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

require_once( dirname( __FILE__ ) . '/vendor/autoload.php' );

if ( ! class_exists( 'TechXelaLatePointGoogleMeet' ) ):

	/**
	 * Main Addon Class.
	 *
	 */

	final class TechXelaLatePointGoogleMeet extends \TechXela\LatePointAddons\MeetingsAddon {
		public $productId = '6E90D023';
		public $version = '9.1.0';
		public $dbVersion = '1.0.4';
		public $addonSlug = 'google-meet';
		public $addonName = 'google-meet-latepoint';
		public $meetingSystemCode = 'gmeet';
		public $meetingSystemName = 'Google Meet';

		protected function defineConstants() {
			$this->define( 'TECHXELA_LATEPOINT_GMEET_ABSPATH', dirname( __FILE__ ) . '/' );
			$this->define( 'TECHXELA_LATEPOINT_GMEET_VIEWS_ABSPATH', TECHXELA_LATEPOINT_GMEET_ABSPATH . 'lib/views/' );
			$this->define( 'TECHXELA_LATEPOINT_GMEET_DB_VERSION', $this->dbVersion );
			$this->define( 'TECHXELA_LATEPOINT_GMEET_IMG_ENTRY_POINTS_URL', $this->publicImgUrl() . 'entry-points/' );
		}

		protected function meetingsAddonIncludes() {
			include_once( TECHXELA_LATEPOINT_GMEET_ABSPATH . 'lib/helpers/booking_helper.php' );
			include_once( TECHXELA_LATEPOINT_GMEET_ABSPATH . 'lib/helpers/customer_helper.php' );
			include_once( TECHXELA_LATEPOINT_GMEET_ABSPATH . 'lib/helpers/database_helper.php' );
			include_once( TECHXELA_LATEPOINT_GMEET_ABSPATH . 'lib/helpers/meeting_helper.php' );
			include_once( TECHXELA_LATEPOINT_GMEET_ABSPATH . 'lib/helpers/service_helper.php' );
			include_once( TECHXELA_LATEPOINT_GMEET_ABSPATH . 'lib/helpers/settings_helper.php' );
		}

		protected function meetingsAddonInitHooks() {
			add_filter( 'latepoint_addons_sqls', [ TechXelaLatePointGoogleMeetDatabaseHelper::class, 'addonsSqls' ] );
		}

		protected function meetingsAddonLatePointInit() {
			TechXelaLatePointGoogleMeetBookingHelper::$meetingSystemCode = $this->meetingSystemCode;
			TechXelaLatePointGoogleMeetBookingHelper::$meetingSystemName = $this->meetingSystemName;

			TechXelaLatePointGoogleMeetServiceHelper::$meetingSystemCode = $this->meetingSystemCode;
			TechXelaLatePointGoogleMeetServiceHelper::$meetingSystemName = $this->meetingSystemName;
		}

		protected function meetingsAddonSystemSettings() {
			TechXelaLatePointGoogleMeetSettingsHelper::externalMeetingSystemSettings();
		}

		protected function meetingsAddonServiceFormAfter( $service ) {
			TechXelaLatePointGoogleMeetServiceHelper::serviceFormAfter( $service );
		}

		protected function meetingsAddonServiceSaved( \OsServiceModel $service, $isNewRecord, $serviceParams ) {
			TechXelaLatePointGoogleMeetServiceHelper::serviceSaved( $service, $isNewRecord, $serviceParams );
		}

		protected function meetingsAddonBookingCreated( $booking ) {
			TechXelaLatePointGoogleMeetBookingHelper::bookingCreated( $booking );
		}

		protected function meetingsAddonBookingUpdated( $booking, $oldBooking ) {
			TechXelaLatePointGoogleMeetBookingHelper::bookingUpdated( $booking, $oldBooking );
		}

		protected function meetingsAddonBookingWillBeDeleted( $bookingId ) {
			TechXelaLatePointGoogleMeetBookingHelper::bookingWillBeDeleted( $bookingId );
		}

		protected function meetingsAddonBookingQuickFormAfter( $booking ) {
			TechXelaLatePointGoogleMeetBookingHelper::bookingQuickFormAfter( $booking );
		}

		protected function meetingsAddonAvailableVarsBooking() {
			TechXelaLatePointGoogleMeetBookingHelper::availableVarsBooking();
		}

		protected function meetingsAddonReplaceBookingVars( $text, $booking, $originalText, $needles, $replacements ) {
			return TechXelaLatePointGoogleMeetBookingHelper::replaceBookingVars($text, $booking, $originalText, $needles, $replacements);
		}

		protected function meetingsAddonCustomerDashboardAfterBookingInfoTile( $booking ) {
			TechXelaLatePointGoogleMeetCustomerHelper::customerDashboardAfterBookingInfoTile( $booking );
		}
	}

endif;

techxela_latepoint_addons_maybe_init( 'TechXelaLatePointGoogleMeet',
	[
		'latepoint-google-calendar/latepoint-google-calendar.php' => [
			'name' => 'LatePoint Google Calendar Addon',
			'link' => 'https://latepoint.com/addons/2'
		]
	] );
