<?php 

class OsPaymentsSquareupHelper {
  public static $processor_name = 'squareup';
  public static $default_currency_iso_code = 'USD';
  public static $default_country_code = 'US';

  public static $squareup = false;

  public static function get_environment_name(){
    $environment_name = (OsSettingsHelper::is_env_payments_live()) ? \Square\Environment::PRODUCTION : \Square\Environment::SANDBOX;
    return $environment_name;
  }

  public static function get_currency_iso_code(){
    return OsSettingsHelper::get_settings_value('squareup_currency_iso_code', self::$default_currency_iso_code);
  }

	public static function get_publishable_key(){
		return OsSettingsHelper::get_settings_value('squareup_publishable_key', '');
	}

	public static function get_location_id(){
		return OsSettingsHelper::get_settings_value('squareup_location_id', '');
	}

	public static function get_secret_key(){
		return OsSettingsHelper::get_settings_value('squareup_secret_key');
	}

	public static function set_api_key(){
		if(self::get_secret_key()){
      self::$squareup = new \Square\SquareClient(['accessToken' => self::get_secret_key(), 'environment' => self::get_environment_name() ]);
		}
	}


  public static function get_customer($squareup_customer_id){
    $customersApi = self::$squareup->getCustomersApi();
    $apiResponse = $customersApi->retrieveCustomer($squareup_customer_id);

    if ($apiResponse->isSuccess()) {
      return $apiResponse->getResult()->getCustomer();
    } else {
      $errors = $apiResponse->getErrors();

      $error_messages = [];
      foreach($errors as $error){
        $error_messages[] = $error->getDetail();
      }
      return new WP_Error('get_customer_error', implode(', ', $error_messages));
    }
  }

  private static function get_properties_allowed_to_update($roles = 'admin'){
    return array('email', 'first_name', 'last_name', 'phone');
  }

  public static function update_customer($squareup_customer_id, $customer){
    $customersApi = self::$squareup->getCustomersApi();

    $body = new Square\Models\UpdateCustomerRequest;
    $body->setGivenName($customer->first_name);
    $body->setFamilyName($customer->last_name);
    $body->setEmailAddress($customer->email);
    $body->setPhoneNumber($customer->phone);

    $apiResponse = $customersApi->updateCustomer($squareup_customer_id, $body);

    if ($apiResponse->isSuccess()) {
        return $apiResponse->getResult()->getCustomer();
    } else {
      $errors = $apiResponse->getErrors();
      $error_messages = [];
      foreach($errors as $error){
        $error_messages[] = $error->getDetail();
      }
      return new WP_Error('get_customer_error', implode(', ', $error_messages));
    }
  }

  public static function create_customer($customer){
    $customersApi = self::$squareup->getCustomersApi();

    $body = new Square\Models\CreateCustomerRequest;
    $body->setGivenName($customer->first_name);
    $body->setFamilyName($customer->last_name);
    $body->setEmailAddress($customer->email);
    $body->setPhoneNumber($customer->phone);
    $body->setReferenceId($customer->id);

    $apiResponse = $customersApi->createCustomer($body);

    if ($apiResponse->isSuccess()) {
      return $apiResponse->getResult()->getCustomer();
    } else {
      $errors = $apiResponse->getErrors();

      $error_messages = [];
      foreach($errors as $error){
        $error_messages[] = $error->getDetail();
      }
      return new WP_Error('create_customer_error', implode(', ', $error_messages));
    }
  }


  public static function is_zip_code_removed(){
    return OsSettingsHelper::is_on('squareup_remove_zip_code', false);
  }


  public static function create_charge($squareup_customer_id, $payment_token, $amount, $idempotency_key, $description = false, $payment_verification_token = false){
    $paymentsApi = self::$squareup->getPaymentsApi();

    $body_amountMoney = new Square\Models\Money;
    $body_amountMoney->setAmount($amount);
    $body_amountMoney->setCurrency(self::get_currency_iso_code());

    $body = new Square\Models\CreatePaymentRequest(
        $payment_token,
        $idempotency_key,
        $body_amountMoney
    );
    $body->setCustomerId($squareup_customer_id);

		if($payment_verification_token) $body->setVerificationToken($payment_verification_token);
		if($description) $body->setNote($description);

    $apiResponse = $paymentsApi->createPayment($body);

    if ($apiResponse->isSuccess()) {
      $responseResult = $apiResponse->getResult();
      if($responseResult->getErrors()){
        $error_messages = [];
        $errors = $responseResult->getErrors();
        foreach($errors as $error){
          $error_messages[] = $error->getDetail();
        }
        return new WP_Error('create_customer_error', implode(', ', $error_messages));
      }else{
        return $responseResult->getPayment();
      }
    } else {
      $errors = $apiResponse->getErrors();

      $error_messages = [];
      foreach($errors as $error){
        $error_messages[] = $error->getDetail();
      }
      return new WP_Error('create_charge_error', implode(', ', $error_messages));
    }
  }

	public static function create_payment_link(){
		// TODO : https://developer.squareup.com/docs/checkout-api/quick-pay-checkout

		$price_money = new \Square\Models\Money();
		$price_money->setAmount(10000);
		$price_money->setCurrency('USD');

		$quick_pay = new \Square\Models\QuickPay(
		    'Auto Detailing',
		    $price_money,
		    'A9Y43N9ABXZBP'
		);

		$body = new \Square\Models\CreatePaymentLinkRequest();
		$body->setIdempotencyKey('cd9e25dc-d9f2-4430-aedb-61605070e95f');
		$body->setQuickPay($quick_pay);

		$api_response = $client->getCheckoutApi()->createPaymentLink($body);

		if ($api_response->isSuccess()) {
		    $result = $api_response->getResult();
		} else {
		    $errors = $api_response->getErrors();
		}
	}

  public static function charge_by_token($payment_token, $booking, $customer, $payment_verification_token = false){
    $result = ['message' => '', 'status' => ''];
    if(isset($payment_token) && !empty($payment_token)){
      $squareup_customer_id = $customer->get_meta_by_key('squareup_customer_id');
      if($squareup_customer_id){
        // has squareup customer id
        $squareup_customer = self::update_customer($squareup_customer_id, $customer);
      }else{
        // does not have squareup customer id
        $squareup_customer = self::create_customer($customer);
        if(is_wp_error($squareup_customer)){
          $result['status'] = LATEPOINT_STATUS_ERROR;
          $result['message'] = $squareup_customer->get_error_message();
          return $result;
        }
        $squareup_customer_id = $squareup_customer->getId();
        $customer->save_meta_by_key('squareup_customer_id', $squareup_customer_id);
      }

      // TODO replace this key with the one sent from the frontend and embedded in the $booking object
      $idempotency_key = OsUtilHelper::generate_uuid();
			$description = $booking->service->name.': '.$booking->agent->full_name.', '.$booking->nice_start_datetime;
      $squareup_charge = self::create_charge($squareup_customer_id, $payment_token, $booking->specs_calculate_price_to_charge(LATEPOINT_PAYMENT_METHOD_CARD), $idempotency_key, $description, $payment_verification_token);
      if(is_wp_error($squareup_charge)){
        $result['status'] = LATEPOINT_STATUS_ERROR;
        $result['message'] = $squareup_charge->get_error_message();
        return $result;
      }
      $result['charge_id'] = $squareup_charge->getId();
      $result['message'] = __('Payment was processed successfully', 'latepoint-payments-squareup');
      $result['status'] = LATEPOINT_STATUS_SUCCESS;
      $result['funds_status'] = ($squareup_charge == 'COMPLETED') ? LATEPOINT_TRANSACTION_FUNDS_STATUS_CAPTURED : LATEPOINT_TRANSACTION_FUNDS_STATUS_AUTHORIZED;
    }else{
      $result['status'] = LATEPOINT_STATUS_ERROR;
      $result['message'] = __('Payment Token Missing', 'latepoint-payments-squareup');
    }
    return $result;
  }


  public static function verify_transaction($payment_token){
    try{
      // verify using the library
    } catch(Exception $e){
      $transaction = false;
      error_log('Squareup Error: '. $e->getMessage());
    }
    return $transaction;
  }

  public static function zero_decimal_currencies_list(){
    return array();
  }

  public static function convert_charge_amount_to_requirements($charge_amount){
    $iso_code = self::get_currency_iso_code();
    if(in_array($iso_code, self::zero_decimal_currencies_list())){
      return round($charge_amount);
    }else{
      $number_of_decimals = OsSettingsHelper::get_settings_value('number_of_decimals', '2');
      return number_format((float)$charge_amount, $number_of_decimals, '.', '') * pow(10, $number_of_decimals);
    }
  }

  public static function load_countries_list(){
  	$country_codes = ['AU' => 'Australia',
                      'CA' => 'Canada',
                      'JP' => 'Japan',
                      'GB' => 'United Kingdom',
                      'US' => 'United States'];
  	return $country_codes;
  }

  public static function load_currencies_list(){
    return ["AED" => "United Arab Emirates dirham",
            "AFN" => "Afghan afghani",
            "ALL" => "Albanian lek",
            "AMD" => "Armenian dram",
            "ANG" => "Netherlands Antillean guilder",
            "AOA" => "Angolan kwanza",
            "ARS" => "Argentine peso",
            "AUD" => "Australian dollar",
            "AWG" => "Aruban florin",
            "AZN" => "Azerbaijani manat",
            "BAM" => "Bosnia and Herzegovina convertible mark",
            "BBD" => "Barbados dollar",
            "BDT" => "Bangladeshi taka",
            "BGN" => "Bulgarian lev",
            "BHD" => "Bahraini dinar",
            "BIF" => "Burundian franc",
            "BMD" => "Bermudian dollar",
            "BND" => "Brunei dollar",
            "BOB" => "Boliviano",
            "BOV" => "Bolivian Mvdol",
            "BRL" => "Brazilian real",
            "BSD" => "Bahamian dollar",
            "BTN" => "Bhutanese ngultrum",
            "BWP" => "Botswana pula",
            "BYR" => "Belarusian ruble",
            "BZD" => "Belize dollar",
            "CAD" => "Canadian dollar",
            "CDF" => "Congolese franc",
            "CHE" => "WIR Euro",
            "CHF" => "Swiss franc",
            "CHW" => "WIR Franc",
            "CLF" => "Unidad de Fomento",
            "CLP" => "Chilean peso",
            "CNY" => "Chinese yuan",
            "COP" => "Colombian peso",
            "COU" => "Unidad de Valor Real",
            "CRC" => "Costa Rican colon",
            "CUC" => "Cuban convertible peso",
            "CUP" => "Cuban peso",
            "CVE" => "Cape Verdean escudo",
            "CZK" => "Czech koruna",
            "DJF" => "Djiboutian franc",
            "DKK" => "Danish krone",
            "DOP" => "Dominican peso",
            "DZD" => "Algerian dinar",
            "EGP" => "Egyptian pound",
            "ERN" => "Eritrean nakfa",
            "ETB" => "Ethiopian birr",
            "EUR" => "Euro",
            "FJD" => "Fiji dollar",
            "FKP" => "Falkland Islands pound",
            "GBP" => "Pound sterling",
            "GEL" => "Georgian lari",
            "GHS" => "Ghanaian cedi",
            "GIP" => "Gibraltar pound",
            "GMD" => "Gambian dalasi",
            "GNF" => "Guinean franc",
            "GTQ" => "Guatemalan quetzal",
            "GYD" => "Guyanese dollar",
            "HKD" => "Hong Kong dollar",
            "HNL" => "Honduran lempira",
            "HRK" => "Croatian kuna",
            "HTG" => "Haitian gourde",
            "HUF" => "Hungarian forint",
            "IDR" => "Indonesian rupiah",
            "ILS" => "Israeli new shekel",
            "INR" => "Indian rupee",
            "IQD" => "Iraqi dinar",
            "IRR" => "Iranian rial",
            "ISK" => "Icelandic króna",
            "JMD" => "Jamaican dollar",
            "JOD" => "Jordanian dinar",
            "JPY" => "Japanese yen",
            "KES" => "Kenyan shilling",
            "KGS" => "Kyrgyzstani som",
            "KHR" => "Cambodian riel",
            "KMF" => "Comoro franc",
            "KPW" => "North Korean won",
            "KRW" => "South Korean won",
            "KWD" => "Kuwaiti dinar",
            "KYD" => "Cayman Islands dollar",
            "KZT" => "Kazakhstani tenge",
            "LAK" => "Lao kip",
            "LBP" => "Lebanese pound",
            "LKR" => "Sri Lankan rupee",
            "LRD" => "Liberian dollar",
            "LSL" => "Lesotho loti",
            "LTL" => "Lithuanian litas",
            "LVL" => "Latvian lats",
            "LYD" => "Libyan dinar",
            "MAD" => "Moroccan dirham",
            "MDL" => "Moldovan leu",
            "MGA" => "Malagasy ariary",
            "MKD" => "Macedonian denar",
            "MMK" => "Myanmar kyat",
            "MNT" => "Mongolian tögrög",
            "MOP" => "Macanese pataca",
            "MRO" => "Mauritanian ouguiya",
            "MUR" => "Mauritian rupee",
            "MVR" => "Maldivian rufiyaa",
            "MWK" => "Malawian kwacha",
            "MXN" => "Mexican peso",
            "MXV" => "Mexican Unidad de Inversion",
            "MYR" => "Malaysian ringgit",
            "MZN" => "Mozambican metical",
            "NAD" => "Namibian dollar",
            "NGN" => "Nigerian naira",
            "NIO" => "Nicaraguan córdoba",
            "NOK" => "Norwegian krone",
            "NPR" => "Nepalese rupee",
            "NZD" => "New Zealand dollar",
            "OMR" => "Omani rial",
            "PAB" => "Panamanian balboa",
            "PEN" => "Peruvian sol",
            "PGK" => "Papua New Guinean kina",
            "PHP" => "Philippine peso",
            "PKR" => "Pakistani rupee",
            "PLN" => "Polish złoty",
            "PYG" => "Paraguayan guaraní",
            "QAR" => "Qatari riyal",
            "RON" => "Romanian leu",
            "RSD" => "Serbian dinar",
            "RUB" => "Russian ruble",
            "RWF" => "Rwandan franc",
            "SAR" => "Saudi riyal",
            "SBD" => "Solomon Islands dollar",
            "SCR" => "Seychelles rupee",
            "SDG" => "Sudanese pound",
            "SEK" => "Swedish krona",
            "SGD" => "Singapore dollar",
            "SHP" => "Saint Helena pound",
            "SLL" => "Sierra Leonean leone",
            "SOS" => "Somali shilling",
            "SRD" => "Surinamese dollar",
            "SSP" => "South Sudanese pound",
            "STD" => "Sao Tome and Príncipe dobra",
            "SVC" => "Salvadoran colón",
            "SYP" => "Syrian pound",
            "SZL" => "Swazi lilangeni",
            "THB" => "Thai baht",
            "TJS" => "Tajikstani somoni",
            "TMT" => "Turkmenistan manat",
            "TND" => "Tunisian dinar",
            "TOP" => "Tongan pa'anga",
            "TRY" => "Turkish lira",
            "TTD" => "Trinidad and Tobago dollar",
            "TWD" => "New Taiwan dollar",
            "TZS" => "Tanzanian shilling",
            "UAH" => "Ukrainian hryvnia",
            "UGX" => "Ugandan shilling",
            "USD" => "United States dollar",
            "UYI" => "Uruguay Peso en Unidedades Indexadas",
            "UYU" => "Uruguyan peso",
            "UZS" => "Uzbekistan som",
            "VEF" => "Venezuelan bolívar soberano",
            "VND" => "Vietnamese đồng",
            "VUV" => "Vanuatu vatu",
            "WST" => "Samoan tala",
            "XAF" => "CFA franc BEAC",
            "XAG" => "Silver",
            "XAU" => "Gold",
            "XBA" => "European Composite Unit",
            "XBB" => "European Monetary Unit",
            "XBC" => "European Unit of Account 9",
            "XBD" => "European Unit of Account 17",
            "XCD" => "East Caribbean dollar",
            "XDR" => "Special drawing rights (International Monetary Fund)",
            "XOF" => "CFA franc BCEAO",
            "XPD" => "Palladium",
            "XPF" => "CFP franc",
            "XPT" => "Platinum",
            "YER" => "Yemeni rial",
            "ZAR" => "South African rand",
            "ZMK" => "Zambian kwacha",
            "ZMW" => "Zambian kwacha",
            "BTC" => "Bitcoin"];
  }

}